/*
    _________________________________
 -=(_)_______________________________)=-
   /   .   .   . ____  . ___      _/
  /~ /    /   / /     / /   )2006 /
 (~ (____(___/ (____ / /___/     (
  \ ----------------------------- \
   \    lucidnagual@charter.net    \
    \_     ===================      \
     \   -Owner of "The Conjuring"-  \
      \_     ===================     ~\
       )      Lucid's Mega Pack        )
      /~    The Mother Load v1.1     _/
    _/_______________________________/
 -=(_)_______________________________)=-

 */
using System;
using Server.Items;
using Server.Network;
using Server.Spells;
using Server.Mobiles;
using Server.Targeting;
using System.Collections;
using Server.Enums;
using Server.ACC.CM;
using Server.Saves;


namespace Server.Items
{
	public abstract class BaseRanged : BaseMeleeWeapon
	{
		//--<<Advanced Archery Edit>>---------------------[Start 1/4]
		internal BaseRangedModule m_Module;
		public BoltType m_BoltType;
		public ArrowType m_ArrowType;
		public StringStrength m_Strength;
		public PoundsPerPull m_PullWeight;
		
		private static Mobile m_Mobile;
		private static BaseRanged BRanged;
		
		public bool m_HasBowString = false;
		
		public static TimeSpan StringWarningDelay = TimeSpan.FromSeconds( 10.0 );
		public static DateTime m_NextStringWarning;
		public static TimeSpan AmmoWarningDelay = TimeSpan.FromSeconds( 10.0 );
		public static DateTime m_NextAmmoWarning;
		
		public ArrayList m_StoredProperties = new ArrayList();
		public ArrayList m_StoredAmmoInfo = new ArrayList();
		public ArrayList m_StoredHasBowInfo = new ArrayList();
		public ArrayList m_StoredStringInfo = new ArrayList();
		public ArrayList m_StoredPPPInfo = new ArrayList();
		
		[CommandProperty(AccessLevel.GameMaster)]
		public ArrowType ArrowSelection	{ get { return m_ArrowType; } set { m_ArrowType = value; } }
		
		[CommandProperty(AccessLevel.GameMaster)]
		public BoltType BoltSelection { get { return m_BoltType; } set { m_BoltType = value; } }
		
		[CommandProperty(AccessLevel.GameMaster)]
		public StringStrength StringStrengthSelection { get { return m_Strength; } set { m_Strength = value; } }
		
		[CommandProperty(AccessLevel.GameMaster)]
		public PoundsPerPull PullWeightSelection { get { return m_PullWeight; } set { m_PullWeight = value; } }
		
		[CommandProperty( AccessLevel.GameMaster )]
		public bool HasBowString { get{ return m_HasBowString; } set{ m_HasBowString = value; } }
		
		public static readonly TimeSpan PlayerFreezeDuration = TimeSpan.FromSeconds( 3.0 );
		public static readonly TimeSpan NPCFreezeDuration = TimeSpan.FromSeconds( 6.0 );
		//--<<Advanced Archery Edit>>---------------------[End 1/4]
		
		public abstract int EffectID{ get; }
		public abstract Type AmmoType{ get; }
		public abstract Item Ammo{ get; }
		
		public override int DefHitSound{ get{ return 0x234; } }
		public override int DefMissSound{ get{ return 0x238; } }
		
		public override SkillName DefSkill{ get{ return SkillName.Archery; } }
		public override WeaponType DefType{ get{ return WeaponType.Ranged; } }
		public override WeaponAnimation DefAnimation{ get{ return WeaponAnimation.ShootXBow; } }
		
		public override SkillName AccuracySkill{ get{ return SkillName.Archery; } }
		
		public BaseRanged( int itemID ) : base( itemID )
		{
		}
		
		public BaseRanged( Serial serial ) : base( serial )
		{
			BRanged = this;
		}
		
		public static void Initialize()
		{
			Mobile from = m_Mobile;
			//Mobile m = (Mobile)parent;
			//BaseWeapon weapon = m.Weapon as BaseWeapon;
		}
		
		public override TimeSpan OnSwing( Mobile attacker, Mobile defender )
		{
			WeaponAbility a = WeaponAbility.GetCurrentAbility( attacker );
			
			// Make sure we've been standing still for .25/.5/1 second depending on Era
			if ( DateTime.Now > ( attacker.LastMoveTime + TimeSpan.FromSeconds( Core.SE ? 0.25 : (Core.AOS ? 0.5 : 1.0) )) || (Core.AOS && WeaponAbility.GetCurrentAbility( attacker ) is MovingShot) )
			{
				bool canSwing = true;
				
				if ( Core.AOS )
				{
					canSwing = ( !attacker.Paralyzed && !attacker.Frozen );
					
					if ( canSwing )
					{
						Spell sp = attacker.Spell as Spell;
						
						canSwing = ( sp == null || !sp.IsCasting || !sp.BlocksMovement );
					}
				}
				
				if ( canSwing && attacker.HarmfulCheck( defender ) )
				{
					attacker.DisruptiveAction();
					attacker.Send( new Swing( 0, attacker, defender ) );
					
					if ( OnFired( attacker, defender ) )
					{
						if ( CheckHit( attacker, defender ) )
							OnHit( attacker, defender );
						else
							OnMiss( attacker, defender );
					}
				}
				
				attacker.RevealingAction();
				
				return GetDelay( attacker );
			}
			else
			{
				attacker.RevealingAction();
				
				return TimeSpan.FromSeconds( 0.25 );
			}
		}
		
		public override void OnHit( Mobile attacker, Mobile defender )
		{
			//--<<Advanced Archery Edit>>---------------------[Start 2/4]
			if ( attacker == null || defender == null )
				return;
			
			if ( AmmoType == typeof( PoisonArrow ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Deadly);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.40 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Greater);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 100 )
				{
					if ( 0.10 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Deadly);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Greater);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.60 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 70 )
				{
					if ( 0.10 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Greater);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.60 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Lesser);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 50 )
				{
					if ( 0.50 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.50 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Lesser);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 50 )
				{
					defender.ApplyPoison(defender, Poison.Lesser);
					defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
				}
			}
			else if ( AmmoType == typeof( PoisonBolt ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Deadly);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.40 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Greater);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 100 )
				{
					if ( 0.10 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Deadly);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Greater);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.60 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 70 )
				{
					if ( 0.10 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Greater);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.30 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.60 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Lesser);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 50 )
				{
					if ( 0.50 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Regular);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
					else if ( 0.50 > Utility.RandomDouble() )
					{
						defender.ApplyPoison(defender, Poison.Lesser);
						defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
					}
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 50 )
				{
					defender.ApplyPoison(defender, Poison.Lesser);
					defender.FixedParticles(0x3728, 200, 25, 69, EffectLayer.Waist);
				}
			}
			else if ( AmmoType == typeof( ExplosiveArrow ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.FixedParticles(0x36BD, 20, 10, 5044, EffectLayer.Waist);
					defender.PlaySound(0x307);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 0, 100, 0, 0, 0);
				}
				
				if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.FixedParticles(0x36BD, 20, 10, 5044, EffectLayer.Waist);
					defender.PlaySound(0x307);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 0, 100, 0, 0, 0);
				}
				
				if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.FixedParticles(0x36BD, 20, 10, 5044, EffectLayer.Waist);
					defender.PlaySound(0x307);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 0, 100, 0, 0, 0);
				}
			}
			else if ( AmmoType == typeof( ExplosiveBolt ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.FixedParticles(0x36BD, 20, 10, 5044, EffectLayer.Waist);
					defender.PlaySound(0x307);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 0, 100, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.FixedParticles(0x36BD, 20, 10, 5044, EffectLayer.Waist);
					defender.PlaySound(0x307);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 0, 100, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.FixedParticles(0x36BD, 20, 10, 5044, EffectLayer.Waist);
					defender.PlaySound(0x307);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 0, 100, 0, 0, 0);
				}
			}
			else if ( AmmoType == typeof( ArmorPiercingArrow ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.PlaySound(0x56);
					defender.FixedParticles(0x3728, 200, 25, 9942, EffectLayer.Waist);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.PlaySound(0x56);
					defender.FixedParticles(0x3728, 200, 25, 9942, EffectLayer.Waist);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.PlaySound(0x56);
					defender.FixedParticles(0x3728, 200, 25, 9942, EffectLayer.Waist);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 100, 0, 0, 0, 0);
				}
			}
			else if ( AmmoType == typeof( ArmorPiercingBolt ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.PlaySound(0x56);
					defender.FixedParticles(0x3728, 200, 25, 9942, EffectLayer.Waist);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.PlaySound(0x56);
					defender.FixedParticles(0x3728, 200, 25, 9942, EffectLayer.Waist);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.PlaySound(0x56);
					defender.FixedParticles(0x3728, 200, 25, 9942, EffectLayer.Waist);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 100, 0, 0, 0, 0);
				}
			}
			else if ( AmmoType == typeof( FreezeArrow ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.PlaySound(0x204);
					defender.Freeze(defender.Player ? PlayerFreezeDuration : NPCFreezeDuration);
					defender.FixedEffect(0x376A, 9, 32);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 0, 0, 100, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.PlaySound(0x204);
					defender.Freeze(defender.Player ? PlayerFreezeDuration : NPCFreezeDuration);
					defender.FixedEffect(0x376A, 9, 32);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 0, 0, 100, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.PlaySound(0x204);
					defender.Freeze(defender.Player ? PlayerFreezeDuration : NPCFreezeDuration);
					defender.FixedEffect(0x376A, 9, 32);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 0, 0, 100, 0, 0);
				}
			}
			else if ( AmmoType == typeof( FreezeBolt ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.PlaySound(0x204);
					defender.Freeze(defender.Player ? PlayerFreezeDuration : NPCFreezeDuration);
					defender.FixedEffect(0x376A, 9, 32);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 0, 0, 100, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.PlaySound(0x204);
					defender.Freeze(defender.Player ? PlayerFreezeDuration : NPCFreezeDuration);
					defender.FixedEffect(0x376A, 9, 32);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 0, 0, 100, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.PlaySound(0x204);
					defender.Freeze(defender.Player ? PlayerFreezeDuration : NPCFreezeDuration);
					defender.FixedEffect(0x376A, 9, 32);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 0, 0, 100, 0, 0);
				}
			}
			else if ( AmmoType == typeof( LightningArrow ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.PlaySound(1471);
					defender.BoltEffect(0);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.PlaySound(1471);
					defender.BoltEffect(0);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.PlaySound(1471);
					defender.BoltEffect(0);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 100, 0, 0, 0, 0);
				}
			}
			else if ( AmmoType == typeof( LightningBolt ) )
			{
				if( attacker.Skills[SkillName.Archery].Value >= 120 )
				{
					defender.PlaySound(1471);
					defender.BoltEffect(0);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(20, 80), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value >= 60 )
				{
					defender.PlaySound(1471);
					defender.BoltEffect(0);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(10, 40), 100, 0, 0, 0, 0);
				}
				
				else if( attacker.Skills[SkillName.Archery].Value < 60 )
				{
					defender.PlaySound(1471);
					defender.BoltEffect(0);
					attacker.DoHarmful(defender);
					AOS.Damage(defender, attacker, Utility.RandomMinMax(5, 20), 100, 0, 0, 0, 0);
				}
			}
			
			if ( Ammo == null )
				attacker.SendMessage( "You are out of arrows, or may have to choose a different type of arrow by double clicking the bow." );
			//--<<Advanced Archery Edit>>---------------------[End 2/4]
			
			if ( attacker.Player && !defender.Player && ( defender.Body.IsAnimal || defender.Body.IsMonster ) && 0.4 >= Utility.RandomDouble() )
				defender.AddToBackpack( Ammo );
			
			base.OnHit( attacker, defender );
		}
		
		public override void OnMiss( Mobile attacker, Mobile defender )
		{
			if ( attacker.Player && 0.4 >= Utility.RandomDouble() )
				Ammo.MoveToWorld( new Point3D( defender.X + Utility.RandomMinMax( -1, 1 ), defender.Y + Utility.RandomMinMax( -1, 1 ), defender.Z ), defender.Map );
			
			base.OnMiss( attacker, defender );
		}
		
		public virtual bool OnFired( Mobile attacker, Mobile defender )
		{
			//--<<Advanced Archery Edit>>---------------------[Start 3/4]
			Container pack = attacker.Backpack;
			BaseQuiver quiver = attacker.FindItemOnLayer( Layer.MiddleTorso ) as BaseQuiver;
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( attacker ); }
			
			if ( module != null && !module.HasBowString )
			{
				if ( DateTime.Now >= m_NextStringWarning )
				{
					m_NextStringWarning = DateTime.Now + StringWarningDelay;
					attacker.SendMessage( "You need a string to use this bow. See a local fletcher to apply the string." );
					return false;
				}
				else
					return false;
			}
			
			if ( Ammo == null )
			{
				if ( DateTime.Now >= m_NextAmmoWarning )
				{
					m_NextAmmoWarning = DateTime.Now + AmmoWarningDelay;
					attacker.SendMessage( "You are out of ammo." );
					return false;
				}
				else
					return false;
			}
			
			if( attacker.Player && quiver != null && quiver.LowerAmmoCost > Utility.Random( 100 ) )
			{
				attacker.MovingEffect( defender, EffectID, 18, 1, false, false );
				return true;
			}
			
			if( attacker.Player &&
			   ( quiver == null || !quiver.ConsumeTotal( AmmoType, 1 ) ) &&
			   (   pack == null ||   !pack.ConsumeTotal( AmmoType, 1 ) ) )
				return false;
			
			attacker.MovingEffect( defender, EffectID, 18, 1, false, false );
			return true;
			//--<<Advanced Archery Edit>>---------------------[End 3/4]
		}
		
		//--<<Advanced Archery Edit>>---------------------[Start 4/4]
		public override void OnDoubleClick( Mobile from )
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( from ); }
			
			if ( IsChildOf( from.Backpack ) || Parent == from )
			{
				if ( module != null && this is Bow || this is CompositeBow || this is ElvenCompositeLongbow ||
				    this is MagicalShortbow || this is Yumi )
				{
					from.SendMessage( "Please choose which type of arrows you wish to use." );
					from.Target = new BowInternalTarget( this, module );
				}
				
				if ( module != null && this is Crossbow || this is HeavyCrossbow || this is RepeatingCrossbow )
				{
					from.SendMessage( "Please choose which type of bolts you wish to use." );
					from.Target = new CrossbowInternalTarget( this, module );
				}
			}
			
			else
				return;
		}
		
		/*		public override void OnDelete()
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( BRanged.Serial, typeof( BaseRangedModule ) );
			
			if ( module != null )
			{
				module.Delete();
			}
			
			base.OnDelete();
		}*/
		
		public override bool OnEquip( Mobile from )
		{
			m_Mobile = from;
			
			return true;
		}
		
		public override bool CanEquip( Mobile from )
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( from == null ) { return false; }
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( from ); }
			
			if ( from != null && module != null && !module.HasBowString )
			{
				from.SendMessage( "You cannot use that without a string." );
				return false;
			}
			
			if ( !Ethics.Ethic.CheckEquip( from, this ) )
				return false;
			
			if( RequiredRace != null && from.Race != RequiredRace )
			{
				if( RequiredRace == Race.Elf )
					from.SendLocalizedMessage( 1072203 ); // Only Elves may use this.
				else
					from.SendMessage( "Only {0} may use this.", RequiredRace.PluralName );
				
				return false;
			}
			else if ( from.Dex < DexRequirement )
			{
				from.SendMessage( "You are not nimble enough to equip that." );
				return false;
			}
			else if ( from.Str < AOS.Scale( StrRequirement, 100 - GetLowerStatReq() ) )
			{
				from.SendLocalizedMessage( 500213 ); // You are not strong enough to equip that.
				return false;
			}
			else if ( from.Int < IntRequirement )
			{
				from.SendMessage( "You are not smart enough to equip that." );
				return false;
			}
			else if ( !from.CanBeginAction( typeof( BaseWeapon ) ) )
			{
				return false;
			}
			else
			{
				return base.CanEquip( from );
			}
		}
		
		public virtual void CheckStringCondition( Mobile from )
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( from ); }
			
			if ( module.m_HasBowString && module.StringStrengthSelection == StringStrength.NoString )
			{
				module.m_HasBowString = false;
			}
			
			if ( !module.m_HasBowString )
			{
				if ( module.StringStrengthSelection == StringStrength.NoString )
				{
					HasBowString = false;
					//from.SendMessage( "You need a string to use this bow. See a local fletcher to apply the string." );
				}
				else
				{
					module.StringStrengthSelection = StringStrength.NoString;
				}
			}
		}
		
		public virtual bool CheckStringDamage( Mobile attacker, Mobile defender )
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( attacker ); }
			
			if ( module.m_HasBowString )
			{
				if ( module.StringStrengthSelection == StringStrength.VeryWeak )
				{
					if ( .05 > Utility.Random( 100 ) )
					{
						BonusDamage( attacker, defender );
						return true;
					}
					if ( .10 > Utility.Random( 1000 ) ) //1 in a 100 chances of breaking.
					{
						module.HasBowString = false;
						module.StringStrengthSelection = StringStrength.NoString;
						attacker.SendMessage( "Your string just broke." );
					}
					return true;
				}
				else if ( module.StringStrengthSelection == StringStrength.Weak )
				{
					if ( .10 > Utility.Random( 100 ) )
					{
						BonusDamage( attacker, defender );
						return true;
					}
					if ( .09 > Utility.Random( 1000 ) ) //1 in a 110 chances of breaking.
					{
						module.HasBowString = false;
						module.StringStrengthSelection = StringStrength.NoString;
						attacker.SendMessage( "Your string just broke." );
					}
					return true;
				}
				else if ( m_Module.StringStrengthSelection == StringStrength.Sturdy )
				{
					if ( .15 > Utility.Random( 100 ) )
					{
						BonusDamage( attacker, defender );
						return true;
					}
					if ( .08 > Utility.Random( 1000 ) ) //1 in a 125 chances of breaking.
					{
						module.HasBowString = false;
						module.StringStrengthSelection = StringStrength.NoString;
						attacker.SendMessage( "Your string just broke." );
					}
					return true;
				}
				else if ( module.StringStrengthSelection == StringStrength.Strong )
				{
					if ( .20 > Utility.Random( 100 ) )
					{
						BonusDamage( attacker, defender );
						return true;
					}
					if ( .05 > Utility.Random( 1000 ) ) //1 in a 200 chances of breaking.
					{
						module.HasBowString = false;
						module.StringStrengthSelection = StringStrength.NoString;
						attacker.SendMessage( "Your string just broke." );
					}
				}
				else if ( module.StringStrengthSelection == StringStrength.Dependable )
				{
					if ( .25 > Utility.Random( 100 ) )
					{
						BonusDamage( attacker, defender );
						return true;
					}
					if ( .03 > Utility.Random( 1000 ) ) //1 in a 333 chances of breaking.
					{
						module.HasBowString = false;
						module.StringStrengthSelection = StringStrength.NoString;
						attacker.SendMessage( "Your string just broke." );
					}
					return true;
				}
				else if ( module.StringStrengthSelection == StringStrength.Indestructable )
				{
					if ( .33 > Utility.Random( 100 ) )
					{
						BonusDamage( attacker, defender );
						return true;
					}
					//No chance of breaking.
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
			
			return false;
		}
		
		public virtual bool BonusDamage( Mobile attacker, Mobile defender )
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( attacker ); }
			
			attacker.SendMessage( "" );
			attacker.SendMessage( 0x35, "** Bonus Hit **" );
			attacker.SendMessage( "" );
			attacker.SendMessage( 0x35, "The strength of your bow and your immpecable skill have given you a perfect hit." );
			attacker.SendMessage( "" );
			
			defender.Say( "* Ouch, that hurt! *" );
			defender.PlaySound( 315 );
			
			if ( module.HasBowString )
			{
				if ( module.m_PullWeight == PoundsPerPull.Fourty )
				{
					attacker.DoHarmful( defender );
					AOS.Damage( defender, attacker, Utility.RandomMinMax( 2, 5 ), 100, 0, 0, 0, 0 );
					return true;
				}
				else if ( module.m_PullWeight == PoundsPerPull.Sixty )
				{
					attacker.DoHarmful( defender );
					AOS.Damage( defender, attacker, Utility.RandomMinMax( 6, 10 ), 100, 0, 0, 0, 0 );
					return true;
				}
				else if ( module.m_PullWeight == PoundsPerPull.Eighty )
				{
					attacker.DoHarmful( defender );
					AOS.Damage( defender, attacker, Utility.RandomMinMax( 11, 15 ), 100, 0, 0, 0, 0 );
					return true;
				}
				else if ( module.m_PullWeight == PoundsPerPull.Hundred )
				{
					attacker.DoHarmful( defender );
					AOS.Damage( defender, attacker, Utility.RandomMinMax( 16, 20 ), 100, 0, 0, 0, 0 );
					return true;
				}
				else
				{
					return false;
				}
			}
			else
				return true;
		}
		
		public virtual bool OnModification( Mobile from )
		{
			double fletching = from.Skills[ SkillName.Fletching ].Base;
			
			if ( from == null ) { return false; }
			
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module == null ) { SaveBaseRangedInfo.ModuleCheck( from ); }
			
			if ( fletching < 100.0 )
			{
				from.SendMessage( "Only a grandmaster bowcrafter can modify this weapon." );
				return false;
			}
			else
			{
				if ( fletching > 104.9 && fletching < 112 )
				{
					module.PullWeightSelection = PoundsPerPull.Sixty;
					this.InvalidateProperties();
					return true;
				}
				else if ( fletching > 111.9 && fletching < 119.9 )
				{
					module.PullWeightSelection = PoundsPerPull.Eighty;
					this.InvalidateProperties();
					return true;
				}
				else if ( fletching > 119.9  )
				{
					module.PullWeightSelection = PoundsPerPull.Hundred;
					this.InvalidateProperties();
					return true;
				}
				else
				{
					module.PullWeightSelection = PoundsPerPull.Fourty;
					this.InvalidateProperties();
					return true;
				}
			}
		}
		
		public virtual Item AmmoArrowSelected()
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module != null )
			{
				switch ( module.m_ArrowType )
				{
					case ArrowType.Normal:
						return new Arrow();
					case ArrowType.Poison:
						return new PoisonArrow();
					case ArrowType.Explosive:
						return new ExplosiveArrow();
					case ArrowType.ArmorPiercing:
						return new ArmorPiercingArrow();
					case ArrowType.Freeze:
						return new FreezeArrow();
					case ArrowType.Lightning:
						return new LightningArrow();
						
					default:
						return new Arrow();
				}
			}
			else
				return new Arrow();
		}
		
		public virtual Type GetArrowSelected()
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module != null )
			{
				switch ( module.m_ArrowType )
				{
					case ArrowType.Normal:
						return typeof( Arrow );
					case ArrowType.Poison:
						return typeof( PoisonArrow );
					case ArrowType.Explosive:
						return typeof( ExplosiveArrow );
					case ArrowType.ArmorPiercing:
						return typeof( ArmorPiercingArrow );
					case ArrowType.Freeze:
						return typeof( FreezeArrow );
					case ArrowType.Lightning:
						return typeof( LightningArrow );
						
					default:
						return typeof( Arrow );
				}
			}
			else
				return typeof( Arrow );
		}
		
		public virtual Item AmmoBoltSelected()
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module != null )
			{
				switch ( module.m_BoltType )
				{
					case BoltType.Normal:
						return new Bolt();
					case BoltType.Poison:
						return new PoisonBolt();
					case BoltType.Explosive:
						return new ExplosiveBolt();
					case BoltType.ArmorPiercing:
						return new ArmorPiercingBolt();
					case BoltType.Freeze:
						return new FreezeBolt();
					case BoltType.Lightning:
						return new LightningBolt();
						
					default:
						return new Bolt();
				}
			}
			else
				return new Bolt();
		}
		
		public virtual Type GetBoltSelected()
		{
			BaseRangedModule module = ( BaseRangedModule )CentralMemory.GetModule( this.Serial, typeof( BaseRangedModule ) );
			
			if ( module != null )
			{
				switch ( module.m_BoltType )
				{
					case BoltType.Normal:
						return typeof( Bolt );
					case BoltType.Poison:
						return typeof( PoisonBolt );
					case BoltType.Explosive:
						return typeof( ExplosiveBolt );
					case BoltType.ArmorPiercing:
						return typeof( ArmorPiercingBolt );
					case BoltType.Freeze:
						return typeof( FreezeBolt );
					case BoltType.Lightning:
						return typeof( LightningBolt );
						
					default:
						return typeof( Bolt );
				}
			}
			else
				return typeof( Bolt );
		}
		
		
		private class BowInternalTarget : Target
		{
			private BaseRanged it_Ranged;
			private BaseRangedModule it_Module;
			
			public BowInternalTarget( BaseRanged ranged, BaseRangedModule module ) : base( 1, false, TargetFlags.None )
			{
				it_Ranged = ranged;
				it_Module = module;
			}
			
			protected override void OnTarget( Mobile from, object targeted )
			{
				Console.WriteLine( "Code making it to OnTarget." );
				
				if ( it_Module != null )
				{
					if ( targeted is Item && targeted is Arrow )
					{
						Item item = ( Item )targeted;
						
						if ( item.GetType() == typeof( Arrow ) )
						{
							it_Module.ArrowSelection = ArrowType.Normal;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( PoisonArrow ) )
						{
							it_Module.ArrowSelection = ArrowType.Poison;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( ExplosiveArrow ) )
						{
							it_Module.ArrowSelection = ArrowType.Explosive;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( ArmorPiercingArrow ) )
						{
							it_Module.ArrowSelection = ArrowType.ArmorPiercing;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( FreezeArrow ) )
						{
							it_Module.ArrowSelection = ArrowType.Freeze;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( LightningArrow ) )
						{
							it_Module.ArrowSelection = ArrowType.Lightning;
							it_Ranged.InvalidateProperties();
						}
						else
							from.SendMessage( "You must select a arrow." );
					}
					else if ( targeted is Item && targeted is BaseStringer )
					{
						Item item = ( Item )targeted;
						Console.WriteLine( "Code making it to BaseStringer check." );
						
						if ( it_Ranged.OnModification( from ) == false )
							return;
						
						Console.WriteLine( "Code making it passed OnModification check." );
						
						if ( it_Module.HasBowString )
						{
							if ( it_Module.StringStrengthSelection == StringStrength.NoString )
								from.SendMessage( "This item is already strung." );
							
							else
								it_Module.HasBowString = false;
						}
						/*else if ( !IsChildOf( from.Backpack ) )
						{
							from.SendLocalizedMessage( 1042001 ); // That must be in your pack for you to use it.
						}*/
						else
						{
							if ( item.GetType() == typeof( LinenThreadStringer ) )
							{
								it_Module.m_Strength = StringStrength.VeryWeak;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( CottonStringer ) )
							{
								it_Module.m_Strength = StringStrength.Weak;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( SilkStringer ) )
							{
								it_Module.m_Strength = StringStrength.Sturdy;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( HorseHairStringer ) )
							{
								it_Module.m_Strength = StringStrength.Strong;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( HempStringer ) )
							{
								it_Module.m_Strength = StringStrength.Dependable;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( AngelHairStringer ) )
							{
								it_Module.m_Strength = StringStrength.Indestructable;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else
								from.SendMessage( "Can Only target stringer items!" );
						}
					}
					else
						from.SendMessage( "You can only target a bolt or bow stringer." );
				}
				else
				{
					SaveBaseRangedInfo.ModuleCheck( from );
					from.SendMessage( "A module is being created for you. Please try again!" );
				}
			}
		}
		
		private class CrossbowInternalTarget : Target
		{
			private BaseRanged it_Ranged;
			private BaseRangedModule it_Module;
			
			public CrossbowInternalTarget( BaseRanged ranged, BaseRangedModule module ) : base( 1, false, TargetFlags.None )
			{
				it_Ranged = ranged;
				it_Module = module;
			}
			
			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( it_Module != null )
				{
					if ( targeted is Item && targeted is Bolt )
					{
						Item item = ( Item )targeted;
						
						if ( item.GetType() == typeof( Bolt ) )
						{
							it_Module.BoltSelection = BoltType.Normal;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( PoisonBolt ) )
						{
							it_Module.BoltSelection = BoltType.Poison;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( ExplosiveBolt ) )
						{
							it_Module.BoltSelection = BoltType.Explosive;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( ArmorPiercingBolt ) )
						{
							it_Module.BoltSelection = BoltType.ArmorPiercing;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( FreezeBolt ) )
						{
							it_Module.BoltSelection = BoltType.Freeze;
							it_Ranged.InvalidateProperties();
						}
						else if ( item.GetType() == typeof( LightningBolt ) )
						{
							it_Module.BoltSelection = BoltType.Lightning;
							it_Ranged.InvalidateProperties();
						}
						else
							from.SendMessage( "You must select a bolt." );
					}
					else if ( targeted is Item && targeted is BaseStringer )
					{
						Item item = ( Item )targeted;
						
						if ( it_Ranged.OnModification( from ) == false )
							return;
						
						if ( it_Module.HasBowString )
						{
							if ( it_Module.StringStrengthSelection == StringStrength.NoString )
								from.SendMessage( "This item is already strung." );
							
							else
								it_Module.HasBowString = false;
						}
						/*else if ( !IsChildOf( from.Backpack ) )
						{
							from.SendLocalizedMessage( 1042001 ); // That must be in your pack for you to use it.
						}*/
						else
						{
							if ( item.GetType() == typeof( LinenThreadStringer ) )
							{
								it_Module.m_Strength = StringStrength.VeryWeak;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( CottonStringer ) )
							{
								it_Module.m_Strength = StringStrength.Weak;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( SilkStringer ) )
							{
								it_Module.m_Strength = StringStrength.Sturdy;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( HorseHairStringer ) )
							{
								it_Module.m_Strength = StringStrength.Strong;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( HempStringer ) )
							{
								it_Module.m_Strength = StringStrength.Dependable;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else if ( item.GetType() == typeof( AngelHairStringer ) )
							{
								it_Module.m_Strength = StringStrength.Indestructable;
								it_Module.m_HasBowString = true;
								it_Ranged.InvalidateProperties();
								item.Delete();
							}
							else
								from.SendMessage( "Can Only Target Stringer Items!" );
						}
					}
					else
						from.SendMessage( "You can only target a bolt or bow stringer." );
				}
				else
				{
					SaveBaseRangedInfo.ModuleCheck( from );
					from.SendMessage( "A module is being created for you. Please try again!" );
				}
			}
		}
		
		public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties(list);
			
			if ( m_Module != null )
			{
				ArrayList strings = new ArrayList();
				
				strings.Add( ( "---------------" ) );
				
				if ( this is Bow || this is CompositeBow || this is ElvenCompositeLongbow ||
				    this is MagicalShortbow || this is Yumi )
				{
					strings.Add( ( "Quiver Using: " + m_Module.m_ArrowType ) );
				}
				
				if ( this is Crossbow || this is HeavyCrossbow || this is RepeatingCrossbow )
				{
					strings.Add( ( "Quiver Using: " + m_Module.m_BoltType ) );
				}
				
				strings.Add( ( "lbs per Pull: " + m_Module.m_PullWeight ) );
				strings.Add( ( "String Str: "  + m_Module.m_Strength ) );
				
				string toAdd = "";
				int amount = strings.Count;
				int current = 1;
				
				foreach ( string str in strings )
				{
					toAdd += str;
					
					if ( current != amount )
						toAdd += "\n";
					
					++current;
				}
				
				if ( toAdd != "" )
					list.Add( 1070722, toAdd );
			}
			else
			{
				return;
			}
		}
		//--<<Advanced Archery Edit>>---------------------[End 4/4]
		
		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 2 ); // version
		}
		
		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
			
			switch ( version )
			{
				case 2:
				case 1:
					{
						break;
					}
				case 0:
					{
						/*m_EffectID =*/ reader.ReadInt();
						break;
					}
			}
			
			if ( version < 2 )
			{
				WeaponAttributes.MageWeapon = 0;
				WeaponAttributes.UseBestSkill = 0;
			}
		}
	}
}
